@@ -9,7 +9,7 @@ from ._version import __version__
 __all__ = ['lifelib_dir', 'compile_rules', 'load_rules', 'reset_tree']
 
 lifelib_dir = os.path.dirname(os.path.abspath(__file__))
-
+this_python = os.path.abspath(sys.executable)
 cygwin_dirs = []
 
 
@@ -104,11 +104,16 @@ def install_cygwin(root_directory=None, packages=['python2', 'python2-numpy', 'p
 
 def verify_installation():
 
+    from .pythlib.lowlevel import regstring
+
     print("Verifying installation...")
     subprocess.check_call(get_local_bash() + ['echo', 'SUCCESS'])
 
     print("Verifying numpy is installed...")
-    subprocess.check_call(get_local_bash() + ['python', '-c', 'import numpy as np; print(np.__version__)'])
+    np_version = subprocess.check_output(get_local_bash() + ['python', '-c', 'import numpy as np; print(np.__version__)'])
+    np_version = regstring(np_version).strip()
+    print("Numpy version: %s" % np_version)
+    return np_version
 
 
 def get_local_bash():
@@ -119,6 +124,14 @@ def get_local_bash():
         return ['bash', os.path.join(lifelib_dir, 'cygbash.sh')]
 
 
+def get_local_python():
+
+    if (os.name == 'nt'):
+        return 'python'
+    else:
+        return this_python
+
+
 def get_compiler():
 
     if (os.name == 'nt'):
@@ -153,7 +166,7 @@ def compile_rules(*rules, **kwargs):
     elif verify_version:
         from .pythlib import Session
         local_bash = get_local_bash()
-        local_python = kwargs.get('local_python', 'python')
+        local_python = kwargs.get('local_python', get_local_python())
         sess = Session(soname, rules=rules, local_bash=local_bash, local_python=local_python)
         compiled_version = sess.lifelib.version
         python_version = __version__
@@ -205,8 +218,15 @@ def load_rules(*rules, **kwargs):
     from .pythlib import Session
     use_indirection = (os.name == 'nt') or kwargs.get('force_indirect', False)
     local_bash = get_local_bash() if use_indirection else None
-    local_python = kwargs.get('local_python', 'python')
-    return Session(soname, rules=rules, local_bash=local_bash, local_python=local_python)
+    local_python = kwargs.get('local_python', get_local_python())
+
+    # Choose the default number of planes to accommodate every rule
+    # for which the lifetree has been compiled, including LifeHistory:
+    default_planes = 1 + max([genera.rule_property(r, 'bitplanes') for r in rules])
+    default_planes = max(4, default_planes)
+
+    return Session(soname, rules=rules, local_bash=local_bash,
+                    local_python=local_python, default_planes=default_planes)
 
 
 def write_all_iterators(f, hist, rules, families):
