@@ -416,7 +416,58 @@ namespace apg {
 
     }
 
+    std::vector<bitworld> apg2vec(std::string apgcode) {
+        /*
+        * Taken almost verbatim from the Catagolue source code.
+        */
+
+        std::vector<bitworld> planes(1);
+
+        int x = 0; int y = 0; bool iny = false;
+        for (unsigned int i = 0; i < apgcode.size(); i++) {
+            char c = apgcode[i]; int a = -1;
+            if (c == '_') {
+                x = 0; y = 0;
+                planes.emplace_back();
+            } else if ((c >= 97) && (c < 123)) {
+                a = c - 87;
+            } else if ((c >= 48) && (c < 58)) {
+                a = c - 48;
+            } else if ((c >= 65) && (c < 91)) {
+                a = c - 55;
+            }
+
+            if (a >= 0) {
+                if (iny) {
+                    iny = false; x += a;
+                } else if (a < 32) {
+                    for (int j = 0; j < 5; j++) {
+                        if (a & (1 << j)) {
+                            planes.back().setcell(x, y+j, 1);
+                        }
+                    }
+                    x += 1;
+                } else if (a == 35) {
+                    x = 0; y += 5;
+                } else {
+                    x += (a - 30);
+                    iny = (a == 34);
+                }
+            }
+        }
+
+        return planes;
+    }
+
     std::vector<bitworld> rle2vec(std::string rle) {
+
+        if (rle.find('!') == std::string::npos) {
+            auto p = rle.find('_');
+            if (p != std::string::npos) {
+                return apg2vec(rle.substr(p+1));
+            }
+        }
+
         std::vector<bitworld> planes;
         uint64_t x = 0; uint64_t y = 0; uint64_t count = 0;
         uint64_t colour = 0;
