@@ -112,6 +112,7 @@ namespace apg {
                             for (uint64_t i = 0; i <= period; i++) {
                                 apg::pattern x = unions[0];
                                 unions[0] = unions[0][1];
+                                if ((unions[0]).empty()) { faithful = false; break; }
                                 for (uint64_t j = 1; j < n; j++) {
                                     if ((unions[j]).empty()) { faithful = false; break; }
                                     if ((x & unions[j]).nonempty()) { faithful = false; break; }
@@ -461,7 +462,8 @@ namespace apg {
         }
 
         template<int H>
-        void deeppurge(std::map<std::string, int64_t> &cm, incubator<56, H> &icb, std::string (*adv)(pattern)) {
+        void deeppurge(std::map<std::string, int64_t> &cm, incubator<56, H> &icb, std::string (*adv)(pattern),
+                        bool remove_annoyances, bool remove_gliders) {
 
             uint64_t excess[8] = {0ull};
 
@@ -471,18 +473,18 @@ namespace apg {
                     uint64_t r = sqt->d[y];
                     while (r != 0) {
                         uint64_t x = __builtin_ctzll(r);
-                        int annoyance = icb.isAnnoyance(sqt, x, y);
+                        int annoyance = (remove_annoyances ? icb.isAnnoyance(sqt, x, y) : 0);
                         if (annoyance > 0) {
                             excess[annoyance] += 1;
-                        } else if (icb.isGlider(sqt, x, y) == 0) {
+                        } else if ((!remove_gliders) || (icb.isGlider(sqt, x, y) == 0)) {
                             // TODO: Identify unknown object
                             auto intList = icb.get_component(sqt, x, y);
                             int population = intList.back();
                             int ll = intList.size() - 1;
                             if (population > 0) {
-                                if (population == 3) {
+                                if ((remove_annoyances) && (population == 3)) {
                                     excess[3] += 1;
-                                } else if (population == 5) {
+                                } else if ((remove_annoyances) && (population == 5)) {
                                     if (ll == 15) {
                                         excess[7] += 1;
                                     } else {
