@@ -561,6 +561,7 @@ namespace apg {
             hypernode<I> hnode2 = hnode;
             uint64_t i = n;
             while (i --> 0) {
+                if ((hnode2.index == 0) && (hnode2.index2 == 0)) { return hypernode<I>(0, hnode.depth - n); }
                 uint64_t tx = (x >> i) & 1;
                 uint64_t ty = (y >> i) & 1;
                 hnode2 = getchild(hnode2, tx + 2 * ty);
@@ -568,13 +569,13 @@ namespace apg {
             return hnode2;
         }
 
-        uint64_t bound_recurse(hypernode<I> hnode, int direction, std::map<std::pair<I, uint32_t>, uint64_t> *memmap) {
+        uint64_t bound_recurse(hypernode<I> hnode, int direction, std::map<std::pair<I, uint32_t>, uint64_t> *memmap, uint32_t pixelsize) {
 
             if (hnode.index2 != 0) {
-                return bound_recurse(breach(hnode), direction, memmap);
+                return bound_recurse(breach(hnode), direction, memmap, pixelsize);
             }
 
-            uint64_t z = (direction & 2) ? 0 : (16ull << hnode.depth);
+            uint64_t z = (direction & 2) ? 0 : ((uint64_t) -1);
             auto it = memmap->find(std::make_pair(hnode.index, hnode.depth));
 
             if (it != memmap->end()) {
@@ -608,9 +609,14 @@ namespace apg {
                         return (tilea ? uint64_left(tilea) : (8 + uint64_left(tileb)));
                     }
                 }
+            } else if (hnode.depth + 4 == pixelsize) {
+                return 0;
             } else {
                 bool nonzero = false;
                 uint64_t n = (hnode.depth < 4) ? hnode.depth : 4;
+                if (hnode.depth + 4 < pixelsize + n) {
+                    n = hnode.depth + 4 - pixelsize;
+                }
                 int64_t nexp = (1ull << n) - 1;
                 if (direction == 0) {
                     for (int64_t x = 0; x <= nexp; x++) {
@@ -618,7 +624,8 @@ namespace apg {
                             hypernode<I> hnode2 = subnode(hnode, x, y, n);
                             if (hnode2.index) {
                                 nonzero = true;
-                                uint64_t a = bound_recurse(hnode2, direction, memmap) + (x << (hnode2.depth + 4));
+                                uint64_t a = bound_recurse(hnode2, direction, memmap, pixelsize);
+                                if (hnode2.depth + 4 - pixelsize < 64) { a += (x << (hnode2.depth + 4 - pixelsize)); }
                                 z = (a < z) ? a : z;
                             }
                         }
@@ -633,7 +640,8 @@ namespace apg {
                             hypernode<I> hnode2 = subnode(hnode, x, y, n);
                             if (hnode2.index) {
                                 nonzero = true;
-                                uint64_t a = bound_recurse(hnode2, direction, memmap) + (y << (hnode2.depth + 4));
+                                uint64_t a = bound_recurse(hnode2, direction, memmap, pixelsize);
+                                if (hnode2.depth + 4 - pixelsize < 64) { a += (y << (hnode2.depth + 4 - pixelsize)); }
                                 z = (a < z) ? a : z;
                             }
                         }
@@ -648,7 +656,8 @@ namespace apg {
                             hypernode<I> hnode2 = subnode(hnode, x, y, n);
                             if (hnode2.index) {
                                 nonzero = true;
-                                uint64_t a = bound_recurse(hnode2, direction, memmap) + (x << (hnode2.depth + 4));
+                                uint64_t a = bound_recurse(hnode2, direction, memmap, pixelsize);
+                                if (hnode2.depth + 4 - pixelsize < 64) { a += (x << (hnode2.depth + 4 - pixelsize)); }
                                 z = (a > z) ? a : z;
                             }
                         }
@@ -663,7 +672,8 @@ namespace apg {
                             hypernode<I> hnode2 = subnode(hnode, x, y, n);
                             if (hnode2.index) {
                                 nonzero = true;
-                                uint64_t a = bound_recurse(hnode2, direction, memmap) + (y << (hnode2.depth + 4));
+                                uint64_t a = bound_recurse(hnode2, direction, memmap, pixelsize);
+                                if (hnode2.depth + 4 - pixelsize < 64) { a += (y << (hnode2.depth + 4 - pixelsize)); }
                                 z = (a > z) ? a : z;
                             }
                         }
@@ -994,7 +1004,8 @@ namespace apg {
         }
 
         hypernode<I> boolean_recurse(hypernode<I> lnode, hypernode<I> rnode, int operation,
-                                        std::map<std::pair<std::pair<I, I>, uint32_t>, I> *memmap) {
+                                        std::map<std::pair<std::pair<I, I>, uint32_t>, I> *memmap,
+                                        std::vector<hypernode<I> > *solids) {
             /*
             *   0 = and
             *   1 = or
@@ -1015,7 +1026,11 @@ namespace apg {
                     return lnode;
                 }
             } else if ((rnode.index2 != 0) || (lnode.index2 != 0)) {
-                return boolean_recurse(breach(lnode), breach(rnode), operation, memmap);
+                return boolean_recurse(breach(lnode), breach(rnode), operation, memmap, solids);
+            } else if ((solids != 0) && (lnode.index == (*solids)[lnode.depth].index)) {
+                if (operation == 0) { return rnode; } else { return lnode; }
+            } else if ((solids != 0) && (rnode.index == (*solids)[rnode.depth].index)) {
+                if (operation == 0) { return lnode; } else { return rnode; }
             } else {
                 // Both operands are nonzero, so we need to actually compute
                 // the result recursively. Firstly, we check to see whether
@@ -1028,13 +1043,13 @@ namespace apg {
                     kiventry<nicearray<I, 4>, I, J >* lptr = ind2ptr_nonleaf(lnode.depth, lnode.index);
                     kiventry<nicearray<I, 4>, I, J >* rptr = ind2ptr_nonleaf(rnode.depth, rnode.index);
                     hypernode<I> ytl = boolean_recurse(hypernode<I>(lptr->key.x[0], lnode.depth-1), hypernode<I>(rptr->key.x[0], rnode.depth-1),
-                                                        operation, memmap);
+                                                        operation, memmap, solids);
                     hypernode<I> ytr = boolean_recurse(hypernode<I>(lptr->key.x[1], lnode.depth-1), hypernode<I>(rptr->key.x[1], rnode.depth-1),
-                                                        operation, memmap);
+                                                        operation, memmap, solids);
                     hypernode<I> ybl = boolean_recurse(hypernode<I>(lptr->key.x[2], lnode.depth-1), hypernode<I>(rptr->key.x[2], rnode.depth-1),
-                                                        operation, memmap);
+                                                        operation, memmap, solids);
                     hypernode<I> ybr = boolean_recurse(hypernode<I>(lptr->key.x[3], lnode.depth-1), hypernode<I>(rptr->key.x[3], rnode.depth-1),
-                                                        operation, memmap);
+                                                        operation, memmap, solids);
                     nicearray<I, 4> cc = {ytl.index, ytr.index, ybl.index, ybr.index};
                     hypernode<I> xcc = make_nonleaf_hn(lnode.depth, cc);
                     memmap->emplace(std::make_pair(std::make_pair(lnode.index, rnode.index), lnode.depth), xcc.index);
@@ -1068,6 +1083,25 @@ namespace apg {
             }
         }
 
+        hypernode<I> boolean_recurse(hypernode<I> lnode, hypernode<I> rnode, int operation,
+                                        std::map<std::pair<std::pair<I, I>, uint32_t>, I> *memmap) {
+
+            std::vector<hypernode<I> > solids;
+
+            nicearray<uint64_t, 4*N> outleaf;
+            std::memset(outleaf.x, 255, 32*N);
+            hypernode<I> hnode(make_leaf(outleaf), 0);
+            solids.push_back(hnode);
+            while (hnode.depth < lnode.depth) {
+                nicearray<I, 4> cc = {hnode.index, hnode.index, hnode.index, hnode.index};
+                hnode = make_nonleaf_hn(hnode.depth + 1, cc);
+                solids.push_back(hnode);
+            }
+
+            return boolean_recurse(lnode, rnode, operation, memmap, ((operation <= 1) ? (&solids) : 0));
+
+        }
+
         uint64_t digest_recurse(hypernode<I> hnode, std::map<std::pair<I, uint32_t>, uint64_t> *memmap) {
 
             if (hnode.index2 != 0) {
@@ -1158,7 +1192,7 @@ namespace apg {
                                 int b = (i & 2) + (j & 2) + (k & 2);
                                 int c = ((b & 4) | (a & 2)) >> 1;
                                 int d = ((b & 2) | (a & 1));
-                                pconvs[c][d] = boolean_recurse(pconvs[c][d], hijk, op, memmap1);
+                                pconvs[c][d] = boolean_recurse(pconvs[c][d], hijk, op, memmap1, 0);
                             }
                         }
                     }
