@@ -93,6 +93,8 @@ namespace apg {
         virtual hypernode<I> pyramid_down(hypernode<I> hnode) = 0;
         virtual hypernode<I> pyramid_up(hypernode<I> hnode) = 0;
 
+        virtual hypernode<I> subnode(hypernode<I> hnode, uint64_t x, uint64_t y, uint64_t n) = 0;
+
         virtual hypernode<I> demorton_recurse(std::map<uint64_t, uint64_t>::iterator &it,
                                               std::map<uint64_t, uint64_t>::iterator &pe,
                                               uint64_t lm, uint64_t upto, uint32_t depth) = 0;
@@ -341,11 +343,15 @@ namespace apg {
             return advance(hnode_initial, rule, steps, 0);
         }
 
-        virtual uint64_t bound_recurse(hypernode<I> hnode, int direction, std::map<std::pair<I, uint32_t>, uint64_t> *memmap) = 0;
+        virtual uint64_t bound_recurse(hypernode<I> hnode, int direction, std::map<std::pair<I, uint32_t>, uint64_t> *memmap, uint32_t pixelsize) = 0;
+
+        uint64_t bound_recurse(hypernode<I> hnode, int direction, int pixelsize) {
+            std::map<std::pair<I, uint32_t>, uint64_t> memmap;
+            return bound_recurse(hnode, direction, &memmap, pixelsize);
+        }
 
         uint64_t bound_recurse(hypernode<I> hnode, int direction) {
-            std::map<std::pair<I, uint32_t>, uint64_t> memmap;
-            return bound_recurse(hnode, direction, &memmap);
+            return bound_recurse(hnode, direction, 0);
         }
 
         bool getbbox(hypernode<I> hnode, int64_t *bbox) {
@@ -482,7 +488,7 @@ namespace apg {
             lmap[3] = 11;
             lmap[4] = 10;
             lmap[5] = 15;
-            return read_macrocell(f, &lmap, rule);
+            return read_macrocell(f, 0, rule);
         }
 
         hypernode<I> load_macrocell(std::string filename, std::string &rule, int64_t x, int64_t y) {
