@@ -372,6 +372,13 @@ namespace apg {
             * For the latter, use the slightly slower ascertain_period().
             */
 
+            if (empty()) {
+                this->minp = (rulestring[1] == '0') ? 2 : 1;
+                this->dt = this->minp;
+                this->dx = 0;
+                this->dy = 0;
+            }
+
             if (dt != 0) { return advance(0); }
 
             int64_t bbox[4] = {0};
@@ -397,6 +404,8 @@ namespace apg {
 
             while (g < gmax) {
 
+                if (x.empty()) { break; }
+
                 x.getrect(bbox);
                 uint64_t h = x.shift(bbox_orig[0] - bbox[0], bbox_orig[1] - bbox[1]).digest();
                 std::pair<int64_t, int64_t> p(bbox[2], bbox[3]);
@@ -536,6 +545,8 @@ namespace apg {
             * Returns the apgcode of a still-life, oscillator or spaceship:
             */
 
+            if (empty()) { return "xs0_0"; }
+
             std::string rep = "#";
             uint64_t p = ascertain_period();
             basepattern<I> x = advance(0);
