@@ -4,6 +4,23 @@ from numbers import Integral
 from .crt import chinese_remainder, large_primes
 
 
+def randfill(item, shape):
+    '''
+    Create a numpy array of a specified shape with specified probabilities
+    of being each item.
+    '''
+
+    import numpy as np
+    if isinstance(item, float):
+        item = {0: 1.0 - item, 1: item}
+
+    p = np.array(list(item.values()), dtype=float)
+    p = p / np.sum(p)
+    v = np.array(list(item.keys()), dtype=np.uint64)
+
+    return np.random.choice(v, p=p, size=shape)
+
+
 class Pattern(object):
 
     def __init__(self, session, ptr, owner):
@@ -22,8 +39,10 @@ class Pattern(object):
         logdiam = self.lifelib('GetDiameterOfPattern', self.ptr)
         beszel_index = self.lifelib('GetBeszelIndex', self.ptr)
         ulqoma_index = self.lifelib('GetUlqomaIndex', self.ptr)
+        rule = self.getrule()
 
-        return ("<Pattern(logdiam=%d, beszel_index=%d, ulqoma_index=%d) owned by %r>" % (logdiam, beszel_index, ulqoma_index, self.owner))
+        return ("<Pattern(logdiam=%d, beszel_index=%d, ulqoma_index=%d, rule=%s) owned by %r>" % (logdiam,
+            beszel_index, ulqoma_index, rule, self.owner))
 
     def advance(self, numgens, exponent=0):
 
@@ -147,22 +166,12 @@ class Pattern(object):
     __nonzero__ = nonempty
     __bool__ = nonempty
 
-    def stream(self, data_or_filename, tempfile='tempfile'):
-
-        tempfile = os.path.abspath(tempfile)
+    def stream(self, data_or_filename):
 
         if isinstance(data_or_filename, list):
-            with open(tempfile, 'w') as f:
-                for i in range(0, len(data_or_filename), 16):
-                    data = data_or_filename[i:i+16]
-                    f.write('%s\n' % str(data))
-            newptr = self.lifelib('MakeSpaceshipStream', self.ptr, tempfile)
-            try:
-                os.remove(tempfile)
-            except OSError:
-                pass
-        else:
-            newptr = self.lifelib('MakeSpaceshipStream', self.ptr, data_or_filename)
+            data_or_filename = str(data_or_filename)
+
+        newptr = self.lifelib('MakeSpaceshipStream', self.ptr, data_or_filename)
         return Pattern(self.session, newptr, self.owner)
 
     def transform(self, tfm):
@@ -215,6 +224,25 @@ class Pattern(object):
         bbox = self.getrect()
         return self.shift(-(bbox[0] + (bbox[2] // 2)), -(bbox[1] + (bbox[3] // 2)))
 
+    def _getbound(self, direction, pixelsize=None, offset=None):
+
+        if pixelsize is None:
+            logdiam = self.lifelib('GetDiameterOfPattern', self.ptr)
+            pixelsize = (logdiam // 30) * 30
+            offset = -(1 << (logdiam - 1))
+
+        b = int(self.lifelib('GetPatternBound', self.ptr, direction, pixelsize)) % (2 ** 30)
+
+        if (pixelsize == 0):
+            return b + offset
+
+        b = b << pixelsize
+
+        x = self(-b, 0) if (direction % 2 == 0) else self(0, -b)
+        x = x[offset:offset + (1 << pixelsize),:] if (direction % 2 == 0) else x[:,offset:offset + (1 << pixelsize)]
+
+        return b + x._getbound(direction, pixelsize - 30, offset)
+
     def match(self, live, dead=None, halo=None):
 
         if halo is not None:
@@ -352,6 +380,8 @@ class Pattern(object):
         else:
             self.write_rle(filename, file_format=file_format, **kwargs)
 
+    # This seems to have better symmetry with Lifetree.load() so we
+    # include it as an alias for Pattern.write_file():
     save = write_file
 
     def viewer(self, filename=None, width=480, height=480, base64=True,
@@ -446,6 +476,23 @@ class Pattern(object):
         else:
             raise TypeError("operator[] accepts either a generation (int) or a pair of slices")
 
+    def _subpops(self, n, ps):
+
+        if (ps < 4):
+            raise ValueError('log2 of pixel size must be at least 4.')
+
+        logdiam = self.lifelib('GetDiameterOfPattern', self.ptr)
+        il = n + ps
+        if (logdiam > il):
+            self64 = self[-(2**(il-1)):(2**(il-1)), -(2**(il-1)):(2**(il-1))]
+        else:
+            self64 = self
+
+        import numpy as np
+        y = np.zeros((2**n, 2**n), dtype=np.uint64, order='C')
+        y = self.lifelib('GetSubpops', self64.ptr, n, ps, y)
+        return y
+
     def coords(self):
 
         p = self.population
@@ -490,7 +537,22 @@ class Pattern(object):
 
         self -= self[x]
 
-        if isinstance(item, Integral):
+        if isinstance(item, float) or isinstance(item, dict):
+            shape = (x[0].stop - x[0].start, x[1].stop - x[1].start)
+            item = randfill(item, shape)
+
+        if hasattr(item, 'ctypes'):
+            import numpy as np
+            xr = np.arange(x[0].start, x[0].stop)
+            yr = np.arange(x[1].start, x[1].stop)
+            coords = np.ascontiguousarray(np.array(list(map(np.ravel, np.meshgrid(xr, yr)))).T, dtype=np.int64)
+            values = np.ascontiguousarray(np.ravel(item), dtype=np.uint64)
+            if (len(coords) != len(values)):
+                raise ValueError('Incompatible shapes for setcells.')
+
+            self.lifelib('SetCells', self.ptr, len(coords), coords, values)
+            return
+        elif isinstance(item, Integral):
             if (item == 0):
                 return
             onecell = self.owner.pattern("o!", self.getrule())
@@ -501,15 +563,27 @@ class Pattern(object):
 
     def getrect(self):
 
+        if self.empty():
+            # An empty pattern does not possess a well-defined bounding box:
+            return None
+
         logdiam = self.lifelib('GetDiameterOfPattern', self.ptr)
 
         if (logdiam > 60):
-            raise ValueError("Pattern is too large for bounding box calculation.")
-
-        bbox = self.lifelib('GetPatternBox', self.ptr, [1, 4])
+            # Slower bigint solution:
+            bounds = [self._getbound(i) for i in range(4)]
+            bbox = [bounds[0], bounds[1], bounds[2] + 1 - bounds[0], bounds[3] + 1 - bounds[1]]
+        else:
+            # Fast int64 solution:
+            bbox = self.lifelib('GetPatternBox', self.ptr, [1, 4])
 
         return bbox
 
+    @property
+    def bounding_box(self):
+
+        return self.getrect()
+
     @property
     def population(self):
 
@@ -528,3 +602,8 @@ class Pattern(object):
     def period(self):
 
         return self.oscar(eventual_oscillator=False, verbose=False)['period']
+
+    @property
+    def displacement(self):
+
+        return self.oscar(eventual_oscillator=False, verbose=False)['displacement']
