@@ -12,7 +12,34 @@ class TestGoL(unittest.TestCase):
     def setUp(self):
 
         self.sess = lifelib.load_rules('b3s23')
-        self.lt = self.sess.lifetree(memory=1000, n_layers=1)
+        self.lt = self.sess.lifetree(memory=1000) # , n_layers=1)
+
+    def test_stream(self):
+
+        push44 = [0, 109, 90, 93, 91, 90, 95, 91, 91, 138, 157, 96, 90, 120, 91, 97, 107, 90, 90, 93, 188]
+        one_glider = self.lt.pattern('3o$o$bo!')
+        elbow = self.lt.pattern('2o$2o!')
+
+        eg = one_glider(5, 2).stream(push44) + elbow
+        self.assertEqual(eg[4096], elbow(-22, -22))
+
+    def test_empty(self):
+
+        empty_pattern = self.lt.pattern()
+        self.assertEqual(empty_pattern.apgcode, 'xs0_0')
+
+    def test_randfill(self):
+
+        pat = self.lt.pattern()
+        pat[70:200, 50:300] = 0.5
+        self.assertEqual(pat.bounding_box, [70, 50, 130, 250])
+        self.assertAlmostEqual(pat.population, 16250, delta=900) # 10 standard deviations
+
+    def test_bbox(self):
+
+        lidka = self.lt.pattern("bo$obo$bo8$8bo$6bobo$5b2obo2$4b3o!", 'b3s23')
+        rect100 = [23 - (10 ** 100) // 4, 13 - (10 ** 100) // 4, (10 ** 100) // 2 - 206, (10 ** 100) // 2 - 186]
+        self.assertEqual(lidka[10 ** 100].getrect(), rect100)
 
     def test_lidka(self):
 
@@ -27,6 +54,12 @@ class TestGoL(unittest.TestCase):
         self.assertEqual(lidka[1093][3511], lidka[3511][1093])
         self.assertNotEqual(lidka[1093], lidka[3511])
 
+    def test_coe(self):
+
+        coe = self.lt.pattern('xq16_gcbgzvgg826frc')
+        osc = coe.oscar()
+        self.assertEqual(osc['period'], 16)
+
     def test_io(self):
 
         try:
