@@ -46,6 +46,10 @@ if [ "$python_exists" = "false" ]; then
 exit 1
 fi
 
+printf "Removing old .so files...\n"
+rm lifelib/pythlib/*.so
+
+printf "Running tests...\n\n"
 set -e
 
 printf "\n\033[1;36m **** UNIT TESTS ****\033[0m\n"
